from psycopg2.extras import NamedTupleCursor, Json, execute_values
from connection_provider import ConnectionProvider

class DBOperations:
    def __init__(self, provider: ConnectionProvider):
        """
        :param provider: ConnectionProvider 인터페이스를 구현한 인스턴스
        """
        self.provider = provider

    def load_feed_specs_from_db(self):
        """
        feed.feed_specs 테이블에서 피드 스펙(각 피드의 publisher, categories, url)을 조회합니다.
        """
        conn = self.provider.get_connection()
        try:
            cursor = conn.cursor(cursor_factory=NamedTupleCursor)
            cursor.execute("SELECT publisher, categories, url FROM feed.feed_specs;")
            results = cursor.fetchall()
            return results
        except Exception as e:
            raise e
        finally:
            cursor.close()
            self.provider.release_connection(conn)

    def save_rss_feed(self, articles, run_id):
        """
        파싱된 기사 리스트를 feed.feed_data 테이블에 저장합니다.
        중복 방지를 위해 link 컬럼에 UNIQUE 제약 조건과 ON CONFLICT 구문을 사용합니다.
        """
        conn = self.provider.get_connection()
        try:
            cursor = conn.cursor()
            insert_query = """
                INSERT INTO feed.feed_data (data, link, run_id)
                VALUES %s
                ON CONFLICT (link) DO NOTHING;
            """
            values = [ (Json(record), record['metadata']['link'], run_id) for record in articles ]
            execute_values(cursor, insert_query, values, page_size=100)
            conn.commit()
        except Exception as e:
            conn.rollback()
            raise e
        finally:
            cursor.close()
            self.provider.release_connection(conn)

    def create_run_log(self, status="started"):
        """
        feed.feed_run_log 테이블에 실행 정보를 기록하고 새로 생성된 run_id를 반환합니다.
        """
        conn = self.provider.get_connection()
        try:
            cursor = conn.cursor()
            insert_query = """
                INSERT INTO feed.feed_run_log (run_time, status)
                VALUES (now(), %s)
                RETURNING run_id;
            """
            cursor.execute(insert_query, (status,))
            run_id = cursor.fetchone()[0]
            conn.commit()
            return run_id
        except Exception as e:
            conn.rollback()
            raise e
        finally:
            cursor.close()
            self.provider.release_connection(conn)

    def update_run_log_status(self, run_id, status):
        """
        feed.feed_run_log 테이블에서 특정 run_id에 해당하는 실행 기록의 상태를 업데이트합니다.
        """
        conn = self.provider.get_connection()
        try:
            cursor = conn.cursor()
            update_query = "UPDATE feed.feed_run_log SET status = %s WHERE run_id = %s;"
            cursor.execute(update_query, (status, run_id))
            conn.commit()
        except Exception as e:
            conn.rollback()
            raise e
        finally:
            cursor.close()
            self.provider.release_connection(conn)
