from db_operations import DBOperations
from parsers import NewsFeedParser, KhanParser

def parse_rss_feed(db_ops: DBOperations):
    """
    1. 실행 기록(run_log)을 생성하여 run_id를 확보합니다.
    2. 데이터베이스에서 피드 스펙(각 피드의 publisher, categories, url)을 조회합니다.
    3. 각 피드 스펙에 대해 적절한 파서(KhanParser 또는 NewsFeedParser)를 선택하여 기사를 파싱합니다.
    4. 파싱된 기사들을 DB에 저장합니다.
    
    반환값은 필요에 따라 파싱 결과나 처리 결과를 리턴할 수 있지만,
    여기서는 아무것도 반환하지 않습니다.
    """
    # 1. 실행 기록 생성 및 run_id 확보
    run_id = db_ops.create_run_log(status="started")
    
    # 2. 피드 스펙 로드 (예: publisher, categories, url)
    feed_specs = db_ops.load_feed_specs_from_db()
    
    # 3. 각 피드 스펙에 대해 파서를 선택하고 파싱 수행
    for publisher, categories, url in feed_specs:
        if publisher == "경향신문":
            # 예시: 경향신문일 경우 KhanParser 사용
            parser = KhanParser(publisher, categories, url)
        else:
            # 그 외에는 기본 파서 사용 (NewsFeedParser)
            parser = NewsFeedParser(publisher, categories, url)
        
        # 파서를 통해 기사 리스트를 파싱
        articles = parser.parse_articles()
        
        # 4. 파싱된 기사들을 DB에 저장 (run_id를 함께 저장하여 어느 실행에 속하는지 기록)
        db_ops.save_rss_feed(articles, run_id)
    
    # 필요에 따라 run_log 상태 업데이트("completed" 등)도 추가할 수 있습니다.
    db_ops.update_run_log_status(run_id, "complete")


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.INFO)
    from my_feed_parser.adapters.local_postgres_connection_provider import LocalPostgresConnectionProvider
    provider = LocalPostgresConnectionProvider()
    db_ops = DBOperations(provider)
    parse_rss_feed(db_ops)
