import abc
import feedparser
import logging
from typing import List, Dict
from bs4 import BeautifulSoup
import re
from dateutil.parser import parse
from datetime import datetime, timezone

class NewsFeedParser(abc.ABC):
    def __init__(self, publisher: str, categories: str, url: str):
        self.publisher = publisher
        self.categories = categories
        self.url = url

    def parse_articles(self) -> List[Dict]:
        logging.info(f"RSS 피드 수집 시작: {self.publisher} - {self.categories}")
        try:
            # RSS 피드 파싱
            feed = feedparser.parse(self.url)
        except Exception as e:
            logging.error(f"RSS 피드 파싱 중 오류 발생 ({self.url}): {e}")
            return []

        if feed.bozo:
            logging.warning(f"RSS 피드 파싱 중 문제가 발생했습니다 ({self.url}): {feed.bozo_exception}")
            return []
        
        articles = []
        for entry in feed.entries:
            try:
                article = {
                    "text": self.cleanse_text(entry.get('summary', 'No Summary')),
                    "metadata": self.extract_article_metadata(entry)
                }
                articles.append(article)

            except Exception as e:
                logging.error(f"기사 파싱 중 오류 발생:{e}")
        
        logging.info(f"{self.publisher} - {self.categories}에서 총 {len(articles)}개의 기사를 수집했습니다.")
        return articles
    
    def extract_article_metadata(self, entry) -> Dict:
        """Extract common fields like published and category."""
        return {
            'title': self.cleanse_text(entry.get('title', 'No Title')),
            'link': entry.get('link', 'No Link'),
            'category': self.categories,
            'published': self.convert_to_isoformat(entry.get('published', None))
        }
    
    @staticmethod
    def cleanse_text(text):
        if not text or not isinstance(text, str):  # 입력이 None이거나 문자열이 아닌 경우 처리
            return ""
        try:
            soup = BeautifulSoup(text, 'html.parser')
            text = soup.get_text()
            text = re.sub(r'[\r\n]+', ' ', text)  # 개행 문자 제거
            text = re.sub(r'[^A-Za-z0-9가-힣 .,!?]', ' ', text)  # 특수문자 제거
            text = re.sub(r'\s+', ' ', text).strip()  # 다중 공백 제거
            return text[:100].rstrip() + "..." if len(text) > 200 else text
        except Exception as e:
            logging.error(f"텍스트 클렌징 중 오류 발생: {e}")
            return ""
        

    @staticmethod
    def convert_to_isoformat(date_str: str) -> str:
        """Convert a date string to ISO 8601 format (minutes precision) with UTC."""
        if not date_str or not isinstance(date_str, str):
            # 현재 시간을 UTC로 설정하고 분까지만 반환
            return datetime.now().replace(second=0, microsecond=0).isoformat().replace("+00:00", "Z")

        try:
            # 문자열 날짜 파싱
            parsed_date = parse(date_str)

            # 시간대 정보가 없는 경우 UTC로 설정
            if parsed_date.tzinfo is None:
                parsed_date = parsed_date.replace(tzinfo=timezone.utc)

            # UTC로 변환하고 분까지만 반환
            parsed_date = parsed_date.astimezone(timezone.utc).replace(second=0, microsecond=0)
            return parsed_date.isoformat().replace("+00:00", "Z")

        except Exception as e:
            logging.warning(f"날짜 문자열 파싱 중 오류 발생: {e}, 입력 값: {date_str}")
            # 오류 발생 시 현재 시간을 UTC로 반환
            return datetime.now().replace(second=0, microsecond=0).isoformat().replace("+00:00", "Z")
